#include <list>
#include <iostream>
#include "adevs.h"
using namespace adevs;
using namespace std;

static int atomic_number = 0;
static int internal_execs = 0;

class SimpleAtomic: public Atomic<char>
{
	public:
		SimpleAtomic():
		Atomic<char>()
		{
			number = atomic_number++;
		}
		void delta_int()
		{
			internal_execs++;
		}
		void delta_ext(double e, const Bag<char>& xb) { assert(false); }
		void delta_conf(const Bag<char>& xb) { assert(false); }
		double ta() { return 1.0; }
		void output_func(Bag<char>& y){}
		void gc_output(Bag<char>& g){}
		bool model_transition() { return true; }
	private:
		int number;
};
	
class SimpleNetwork: public Network<char>
{
	public:
		SimpleNetwork():
		Network<char>()
		{
			SimpleAtomic* model = new SimpleAtomic();
			model->setParent(this);
			models.push_back(model);
		}
		void getComponents(Set<Devs<char>*>& c)
		{
			list<Devs<char>*>::iterator iter;
			for (iter = models.begin(); iter != models.end(); iter++)
			{
				c.insert(*iter);
			}
		}
		void route(const char& value, Devs<char>* model, Bag<Event<char> >& r){}
		bool model_transition()
		{
			SimpleAtomic* model = new SimpleAtomic();
			model->setParent(this);
			models.push_back(model);
			return false;
		}
		~SimpleNetwork()
		{
			list<Devs<char>*>::iterator iter;
			for (iter = models.begin(); iter != models.end(); iter++)
			{
				delete *iter;
			}
		}
	private:
		list<Devs<char>*> models;
};

int main()
{
	SimpleNetwork* model = new SimpleNetwork();
	Simulator<char>* sim = new Simulator<char>(model);
	while (sim->nextEventTime() < DBL_MAX && atomic_number < 10)
	{
		sim->execNextEvent();
		assert(internal_execs+1 == atomic_number);
		internal_execs = 0;
	}
	return 0;
}
